/// @file       webserver1.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $kur$
/// @date       $2014-02-24$
/// @brief      This code demonstrates how to implement web server on Colibri Module.

#include <windows.h>
#include <stdio.h>
#include <winsock2.h>
#include "webserver1.h"                                         ///< self defined File containing HTML page to be displayed on client side
#include "gpioLib.h"                                            ///< GPIO Library for Colibri Module

#define IP_ADDRESS                 "0.0.0.0"                    ///< IP address of Module
#define PORT_NUMBER                10000                        ///< PORT number used for server
#define LOW                        '0'
#define HIGH                       '1'
#define GPIO                       -1
#define PAGE_SIZE                  2000                         ///< HTML page size
#define BUFFER_SIZE                255
#define REC_BUFFER                 512
#define DEVICE_1                   45                           ///< Device 1 connected to SODIMM Pin 45 at X9 connector
#define DEVICE_2                   55                           ///< Device 2 connected to SODIMM Pin 55 at X9 connector
#define DEVICE_3                   63                           ///< Device 3 connected to SODIMM Pin 63 at X9 connector
#define DEVICE_4                   100                          ///< Device 4 connected to SODIMM Pin 100 at X9 connector
#define PAGE_BUFFER_SIZE           3000
#define DEVICE_STATE_SIZE          10

#pragma comment(lib, "Ws2.lib")                                 ///< linking winsock library

void SetDeviceState(CHAR, CHAR, CHAR, CHAR, CHAR *htmlQuery);   ///< function to control devices attached to pin and update html query

int wmain()
{
    WSADATA windowsSocket;                                      ///< structure contains information about the Windows Sockets implementation
    DWORD retVal = 0;
    INT recvRetVal = 0;
    CHAR device1State = LOW;
    CHAR device2State = LOW;
    CHAR device3State = LOW;
    CHAR device4State = LOW;
    CHAR loopCounter = LOW;
    CHAR htmlQuery[PAGE_SIZE];
    CHAR recBuffer[REC_BUFFER];                                 ///< recBuffer to receive data
    CHAR temporaryString[BUFFER_SIZE];
    SOCKET listening_Socket;                                    ///< socket that is bound to a specific transport service provider
    SOCKET client;
    struct sockaddr_in tServerInfo;                             ///< structure contains information about socket address

    InitGPIOLib();                                              ///< initialize GPIO library for Colibri module

    SetPinAltFn(DEVICE_1, GPIO, DIR_OUT);                       ///< setting GPIO Pin 45 as output
    SetPinAltFn(DEVICE_2, GPIO, DIR_OUT);
    SetPinAltFn(DEVICE_3, GPIO, DIR_OUT);
    SetPinAltFn(DEVICE_4, GPIO, DIR_OUT);

    SetDeviceState(device1State, device2State, device3State, device4State, htmlQuery);   ///< setting all device connected off
    retVal = WSAStartup(0x0101, &windowsSocket);                                         ///< initiating use of ws2.dll(library' used for socket programming) with winsock Version 1.1 
    if (retVal != 0)                                                                     ///< if WSAStartup failed to initialize
    {
        printf("WSAStartup failed with error code: %d\n", retVal);
        exit(1);
    }
    listening_Socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);                        ///< socket creation
    if (listening_Socket == INVALID_SOCKET)
    {
        printf("\nsocket creation failed with error code : %d", WSAGetLastError());
        WSACleanup();
        exit(1);
    }

    /// socket binding
    tServerInfo.sin_family = AF_INET;                                                    ///< tcp/udp socket
    tServerInfo.sin_addr.s_addr = inet_addr(IP_ADDRESS);                                 ///< IP Address of Colibri Module
    tServerInfo.sin_port = htons(PORT_NUMBER);                                           ///< port number:10000 used for communication 
    retVal = bind(listening_Socket,(SOCKADDR*)&tServerInfo, sizeof(struct sockaddr));
    if (retVal == SOCKET_ERROR)
    { 
        printf("\nsocket binding failed with error code : %d", WSAGetLastError());
        WSACleanup();
        exit(1);
    }
    retVal = listen(listening_Socket, 10);                                               ///< put socket in listening state with 10 max connection
    if (retVal == SOCKET_ERROR)
    {
        printf("\nsocket listening failed with error code : %d", WSAGetLastError());
        WSACleanup();
        getchar();
        exit(1);
    }
    while (TRUE)                                                                         ///< loop for processing data continuously
    {
        loopCounter = 0;
        client = accept(listening_Socket, NULL, NULL);                                   ///< waiting for request from client
        if (client == INVALID_SOCKET)
        {
            MessageBox(0, L"Could not accept connection request", L"error",0);
            WSACleanup();
            getchar();
            exit(1);
        }
        recvRetVal = recv(client, recBuffer, REC_BUFFER, 0);                             ///< receiving request from Internet
        recBuffer[retVal] = 0;
        while (TRUE)                                                                     ///< get device status from html query received from server
        {
            if (recBuffer[loopCounter + 16] == ' ')
                break;
            temporaryString[loopCounter] = recBuffer[loopCounter + 16];
            loopCounter++;
        }
        temporaryString[loopCounter] = '\0';
        /// Part of query received from server => "GET /Device.htm?Device1=0&Device2=1&Device3=0&Device4=0 HTTP/1.1
        ///                                                                8         18        28        38    
        device1State = temporaryString[8];
        device2State = temporaryString[18];
        device3State = temporaryString[28];
        device4State = temporaryString[38];

        SetDeviceState(device1State, device2State, device3State, device4State, htmlQuery);        ///< switching of devices

        retVal = send(client, htmlQuery, strlen(htmlQuery) - 1, 0);                               ///< sending to HTML page to internet
        if (retVal == SOCKET_ERROR)
        {
            MessageBox(0, L"could not send message to client", L"error", 0);
            WSACleanup();
            exit(1);
        }

        shutdown(client, SD_BOTH);                                                       ///< closing both sending and receiving of client named socket 
        closesocket(client);                                                             ///< closing client named Socket
    }
    closesocket(listening_Socket);                                                       ///< closing listening_socket named Socket
    WSACleanup();                                                                        ///< releasing use of ws2.dll(library' used for socket programming) with winsock Version 1.1 
    DeInitGPIOLib();                                                                     ///< releasing GPIO library 
    return 0;
}

void SetDeviceState(CHAR device_1, CHAR device_2, CHAR device_3, CHAR device_4, CHAR *htmlQuery)
{
    BOOL retVal = TRUE;
    CHAR device1Unchecked[10] = "unchecked";
    CHAR device1Checked[10] = "checked";
    CHAR device2Unchecked[10] = "unchecked";
    CHAR device2Checked[10] = "checked";
    CHAR device3Unchecked[10] = "unchecked";
    CHAR device3Checked[10] = "checked";
    CHAR device4Unchecked[10] = "unchecked";
    CHAR device4Checked[10] = "checked";

    if (device_1 == HIGH)
    {
        retVal = SetPinLevel(DEVICE_1, TRUE);                                            ///< switching ON 1st Device
        strcpy_s(device1Unchecked, DEVICE_STATE_SIZE, "checked");
        strcpy_s(device1Checked, DEVICE_STATE_SIZE, "unchecked");
    }
    else
    {
        retVal = SetPinLevel(DEVICE_1, FALSE);                                           ///< switching OFF 1st Device
        strcpy_s(device1Unchecked, DEVICE_STATE_SIZE, "unchecked");
        strcpy_s(device1Checked, DEVICE_STATE_SIZE, "checked");
    }
    if (device_2 == HIGH)
    {
        retVal = SetPinLevel(DEVICE_2, TRUE);                                            ///< switching ON 2nd Device
        strcpy_s(device2Unchecked, DEVICE_STATE_SIZE, "checked");
        strcpy_s(device2Checked, DEVICE_STATE_SIZE, "unchecked");
    }
    else
    {
        retVal = SetPinLevel(DEVICE_2, FALSE);                                           ///< switching OFF 2nd Device
        strcpy_s(device2Unchecked, DEVICE_STATE_SIZE, "unchecked");
        strcpy_s(device2Checked, DEVICE_STATE_SIZE, "checked");
    }
    if (device_3 == HIGH)
    {
        retVal = SetPinLevel(DEVICE_3, TRUE);                                            ///< switching ON 3rd Device
        strcpy_s(device3Unchecked, DEVICE_STATE_SIZE, "checked");
        strcpy_s(device3Checked, DEVICE_STATE_SIZE, "unchecked");
    }
    else
    {
        retVal = SetPinLevel(DEVICE_3, FALSE);                                           ///< switching OFF 3rd Device
        strcpy_s(device3Unchecked, DEVICE_STATE_SIZE, "unchecked");
        strcpy_s(device3Checked, DEVICE_STATE_SIZE, "checked");
    }
    if (device_4 == HIGH)
    {
        retVal = SetPinLevel(DEVICE_4, TRUE);                                            ///< switching ON 4th Device
        strcpy_s(device4Unchecked, DEVICE_STATE_SIZE, "checked");
        strcpy_s(device4Checked, DEVICE_STATE_SIZE, "unchecked");
    }
    else
    {
        retVal = SetPinLevel(DEVICE_4, FALSE);                                           ///< switching OFF 4th Device
        strcpy_s(device4Unchecked, DEVICE_STATE_SIZE, "unchecked");
        strcpy_s(device4Checked, DEVICE_STATE_SIZE,"checked");
    }
    /// setting new HTML page query to update page
    sprintf_s(htmlQuery, PAGE_SIZE, STR, device1Unchecked, device1Checked, device2Unchecked, device2Checked, device3Unchecked, device3Checked, device4Unchecked, device4Checked);
}